#! /usr/bin/env python

# Copyright (c) 2006 Andrew Choi.  All rights reserved.
#
# This is emphatically NOT free/GPL software.
#
# The use of the source or any binary form of this software, with or
# without modifications, is permitted only for research, educational,
# and non-commercial purposes.
#
# Redistribution of this software or any of its parts, in source or
# binary form, with or without modification, is prohibited.
#
# There is no restriction on the use of the output generated by this
# software.
#
# Filename: chart-translate.py

__all__ = ['main']

import re
import getopt, sys

from toe.io.toereader import *
from toe.io.toewriter import *

from toe.io.biabreader import *
from toe.io.biabwriter import *

from toe.io.lywriter import *

from toe.io.xmlreader import *
from toe.io.xmlwriter import *

#from toe.pdfwriter import *

BIAB_EXT_RE = re.compile('(sg|mg)([1-9]|[a-n]|u)')
TOE_EXT_RE = re.compile('(mjb|toe)')
XML_EXT_RE = re.compile('xml')
LY_EXT_RE = re.compile('ly')
PDF_EXT_RE = re.compile('pdf')

PATHNAME_RE = re.compile('(.*)[.]([^/]*)$')

def file_type(extension):
    if BIAB_EXT_RE.match(extension):
        return 'biab'
    elif TOE_EXT_RE.match(extension):
        return 'toe'
    elif XML_EXT_RE.match(extension):
        return 'xml'
    elif LY_EXT_RE.match(extension):
        return 'ly'
    elif PDF_EXT_RE.match(extension):
        return 'pdf'
    else:
        raise ValueError('Unknown input file extension: %s' % extension)

def extension_from_type(file_type):
    if file_type == 'biab':
        return 'sg1'
    elif file_type == 'toe':
        return 'mjb'
    elif file_type == 'xml':
        return 'xml'
    elif file_type == 'ly':
        return 'ly'
    elif file_type == 'pdf':
        return 'pdf'
    else:
        raise ValueError('Unknown file type: %s' % file_type)

def file_extension(pathname):
    _m = PATHNAME_RE.match(pathname)
    if not _m or _m.group(0) != pathname:
        raise ValueError('No extension in pathname: %s' % pathname)
    return _m.group(2).lower()

def strip_file_extension(pathname):
    _m = PATHNAME_RE.match(pathname)
    if not _m or _m.group(0) != pathname:
        return pathname
    return _m.group(1)

def default_out_type(exec_name):
    exec_name = strip_file_extension(exec_name)
    
    if exec_name.endswith('-biab'):
        return 'biab'
    elif exec_name.endswith('-toe'):
        return 'toe'
    elif exec_name.endswith('-xml') or exec_name.endswith('-xml-i'):
        return 'xml'
    elif exec_name.endswith('-ly') or exec_name.endswith('-ly-i'):
        return 'ly'
    else:
        return 'toe'

def uses_inivisible_notes_from_exec_name(exec_name):
    exec_name = strip_file_extension(exec_name)

    return exec_name.endswith('-i')

def usage():
    print 'Usage: charttranslate (-i)? (-t type|-o outfile) infile+'
    sys.exit(2)

def translate(in_file, in_type, out_file, out_type, uses_invisible_notes):
    # print in_file, in_type, out_file, out_type
    if in_type == 'biab':
        chart = read_biab_file(in_file)
    elif in_type == 'toe':
        chart = read_toe_file(in_file)
    elif in_type == 'xml':
        chart = read_xml_file(in_file)
    else:
        raise ValueError('Unknown input file type: %s' % in_type)

    if out_type == 'biab':
        if chart['style']['meter'] == 3:
            out_extension = '.sg8'
        elif chart['style']['type'] == 4:
            out_extension = '.sg4'
        else:
            out_extension = '.sg1'
        out_file = strip_file_extension(out_file) + out_extension

    substitions = None
    if out_type == 'biab':
        substitions = write_biab_file(out_file, chart)
    elif out_type == 'toe':
        write_toe_file(out_file, chart)
    elif out_type == 'xml':
        write_xml_file(out_file, chart, uses_invisible_notes)
    elif out_type == 'ly':
        write_ly_file(out_file, chart, uses_invisible_notes)
    elif out_type == 'pdf':
        write_pdf_file(out_file, chart)
    else:
        raise ValueError('Unknown output file type: %s' % out_type)

    if substitions != None and len(substitions) > 0:
        print 'The follow substitions were made:\n%s' % substitions 


def main(argv):
    try:
        opts, args = getopt.getopt(argv[1:], 'it:o:', ['invisible', 'type', 'output'])
    except getopt.GetoptError:
        usage()

    uses_invisible_notes = False
    out_type = None
    out_file = None
    for o, a in opts:
        if o in ['-i', '--invisible']:
            uses_invisible_notes = True
        elif o in ['-t', '--type']:
            if a not in ['biab', 'mjb', 'toe', 'ly', 'xml', 'pdf']:
                print 'Unknown output file type: %s' % a
                sys.exit(1)
            out_type = a
            if out_type == 'mjb':
                out_type = 'toe'
        elif o in ['-o', '--output']:
            out_file = a

    if out_file == None and out_type == None:
        out_type = default_out_type(argv[0])

    if out_file != None and out_type != None:
        print 'Cannot use both -t and -o options (output file extension provides type)'
        usage()

    if uses_inivisible_notes_from_exec_name(argv[0]):
        uses_invisible_notes = True

    if out_file != None:
        if len(args) != 1:
            print 'Cannot use -o option and multiple input files (use -t option instead)'
            usage()
        else:
            in_file = args[0]
            in_type = file_type(file_extension(in_file))
            out_type = file_type(file_extension(out_file))
            translate(in_file, in_type, out_file, out_type, uses_invisible_notes)
    else:
        if not args:
            print 'No input file specified'
            usage()
        
        for in_file in args:
            in_type = file_type(file_extension(in_file))
            out_file = strip_file_extension(in_file) + '.' + extension_from_type(out_type)
            translate(in_file, in_type, out_file, out_type, uses_invisible_notes)

if __name__ == '__main__':
    main(sys.argv)
